//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef CONSOLE_HPP
#define CONSOLE_HPP
//------------------------------------------------------------------------------
#include <SFML/Graphics.hpp>
//------------------------------------------------------------------------------
#include "config.hpp"
//------------------------------------------------------------------------------
#include <list>
#include <map>
#include <string>
#include <streambuf>
#include <iostream>
#include <utility>
#include <sstream>
//------------------------------------------------------------------------------
#include "Utils.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
class Console;
//------------------------------------------------------------------------------
#define BUFFER_SIZE 1024
class ConsoleStreamBuf : public std::streambuf
{
    // Source : // http://www.dreamincode.net/code/snippet2499.htm

public:
    ConsoleStreamBuf(Console *targetConsole, std::streambuf *targetCopyBuf, bool isError);

    virtual int overflow(int c);

    virtual int sync();

protected:
    char m_buffer[BUFFER_SIZE];

    Console *m_targetConsole;
    std::streambuf *m_targetCopyBuf;
    bool m_isError;
};

//http://stackoverflow.com/questions/524524/creating-an-ostream
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
#define CONSOLE_MAX_LINES 200
#define CONSOLE_LINE_MAX_RM 30
/// LUA_HEADER_COPY_ON ///
/** The information console. */
/// LUA_HEADER_COPY_OFF ///
class Console
{
public:
    /** Create a console and enable cout and cerr redirection */
    Console();
    /** Destroy the console and reset cout and cerr to their previous state */
    ~Console();

    /** Draw the console on the screen */
    void Render(sf::RenderWindow &App);

    /** Add some new text to the console */
    void AddText(const char *ptr, bool isError);

    /** Show / hide the console */
    void Show(bool isVisible);

    /** Return true if the information console is visible, false if not */
    bool IsVisible() const {return m_isVisible;}
protected:
    // Back streambuf
    std::streambuf *m_cerrBuf, *m_coutBuf;

    ConsoleStreamBuf *m_consoleBufCerr, *m_consoleBufCout;

    // Content list
    typedef std::list<std::pair<std::string, bool> > ListType;
    ListType m_lines;

    sf::String m_text;
    sf::Font m_font;
    Vector2D m_letterSize;

    // Config : Size
    float m_interLine, m_heightRatio, m_margin;
    bool m_isVisible;
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // CONSOLE_HPP
