//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhRect.hpp"
//------------------------------------------------------------------------------
#include <iostream>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
PhRect::PhRect(Group *parent, int phGroupId, const std::string &name, const Vector2D &size) : PhysicObjectLine(parent, phGroupId, name), m_size(size)
{
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
Vector2D PhRect::GetSize() const
{
    return m_size;
}
//------------------------------------------------------------------------------
void PhRect::SetSize(const Vector2D &size)
{
    m_size = size;
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
std::vector<Vector2D> PhRect::GetAllPoints() const
{
    // FIXME : Optimisation
    // -> Cache point list ?

    float angle = GetGlobalRotation();
    Vector2D pos = GetGlobalPosition();

    std::vector<Vector2D> ptList;
    ptList.resize(4);
    ptList[0] = Vector2D(0, 0);
    ptList[1] = Vector2D(m_size.x, 0);
    ptList[2] = Vector2D(m_size.x, m_size.y);
    ptList[3] = Vector2D(0, m_size.y);

    for(size_t i = 0; i < 4; i++)
    {
        ptList[i] = PointRotate(Vector2D(0, 0), ptList[i], angle);
        ptList[i] += pos;
    }

    return ptList;
}
//------------------------------------------------------------------------------
/*void PhRect::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
