//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Shape.hpp"

#include "MyException.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
Shape::Shape(Group *parent, const std::string &name) : DrawableSF(parent, name)
{

}

//------------------------------------------------------------------------------
void Shape::MakeLine(const Vector2D& P1, const Vector2D& P2, float Thickness, const Color& Col, float Outline, const Color& OutlineCol)
{
    *static_cast<sf::Shape*>(this) = sf::Shape::Line(P1, P2, Thickness, Col, Outline, OutlineCol);
}
//------------------------------------------------------------------------------
void Shape::MakeRectangle(const Vector2D& P1, const Vector2D& P2, const Color& Col, float Outline, const Color& OutlineCol)
{
    *static_cast<sf::Shape*>(this) = sf::Shape::Rectangle(P1, P2, Col, Outline, OutlineCol);
}
//------------------------------------------------------------------------------
void Shape::MakeCircle(const Vector2D& Center, float Radius, const Color& Col, float Outline, const Color& OutlineCol)
{
    *static_cast<sf::Shape*>(this) = sf::Shape::Circle(Center, Radius, Col, Outline, OutlineCol);
}
//------------------------------------------------------------------------------
/** Get a point position */
Vector2D Shape::GetPointPosition(int index) const
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::GetPointPosition(): index '" << index + 1 << "' out of range [1, " << GetNbPoints() << "] !");
    }

    return sf::Shape::GetPointPosition(index);
}
//------------------------------------------------------------------------------
/** Get a point color */
Color Shape::GetPointColor(int index) const
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::GetPointColor(): index '" << index + 1 << "' out of range [1, " << GetNbPoints() << "] !");
    }

    return sf::Shape::GetPointColor(index);
}
//------------------------------------------------------------------------------
/** Get a point outline color */
Color Shape::GetPointOutlineColor(int index) const
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::GetPointOutlineColor(): index '" << index + 1 << "' out of range [1, " << GetNbPoints() << "] !");
    }

    return sf::Shape::GetPointOutlineColor(index);
}
//------------------------------------------------------------------------------
/** Change the position of a point. */
void Shape::SetPointPosition(int index, const Vector2D& Position)
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::SetPointPosition(): index '" << index + 1 << "' out of range [1, " << GetNbPoints() << "] !");
    }

    sf::Shape::SetPointPosition(index, Position);
}
//------------------------------------------------------------------------------
/** Change the color of a point */
void Shape::SetPointColor(int index, const Color& Col)
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::SetPointColor(): index '" << index + 1 << "' out of range [0, " << GetNbPoints() << "] !");
    }

    sf::Shape::SetPointColor(index, Col);
}
//------------------------------------------------------------------------------
/** Change the outline color of a point */
void Shape::SetPointOutlineColor(int index, const Color& OutlineCol)
{
    index--;
    if(index < 0 or index >= int(GetNbPoints()))
    {
        MY_THROW("Shape::SetPointOutlineColor(): index '" << index + 1 << "' out of range [0, " << GetNbPoints() << "] !");
    }

    sf::Shape::SetPointOutlineColor(index, OutlineCol);
}
//------------------------------------------------------------------------------
/** Change the size of a rectangle shape (for compatibility with Sprite). P1 position is kept. */
void Shape::SetSize(Vector2D &size)
{
    if(GetNbPoints() != 4)
    {
        MY_THROW("Shape::SetSize(): shape '" << GetName() << "' doesn't have 4 points !");
    }

    Vector2D p1 = GetPointPosition(1);
    SetPointPosition(2, Vector2D(p1.x + size.x, p1.y));
    SetPointPosition(3, p1 + size);
    SetPointPosition(4, Vector2D(p1.x, p1.y + size.y));
}

//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
