------------------------------------------------------------
-- Bonus base
------------------------------------------------------------

function bonusBase_Create(bonusToClone, toGroup, forcedPos)
	if forcedPos == nil then
		forcedPos = Vector2D(math.random() * screenSize.x, 0)
	end
	
	local enemy = bonusToClone:CloneToGroup(toGroup)
	enemy:SetPosition(forcedPos)
	
	-- Speed
	enemy:SetSpeed(enemy:GetSpeed() * (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	return enemy
end

function bonusBase_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	-- Points
	score = score + enemyGrp:GetUserData("points")
	
	if phEnemy:GetName() == "phBidon" then -- Bidon => explode	
		-- Explosion
		local speedVect = bulletGrp:GetSpeed()
		CreateDebris(gameScreen, hitPos, speedVect, 300)
		CreateExplosion(hitPos, math.pi * 3/5, speedVect)
	
		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		enemyGrp:Destroy() -- We destroy the group
	else -- Plus de parachute
		local phPara = enemyGrp:FindByName("phPara")
		enemyGrp:FindByName("sprPara"):Destroy()
		
		CreateSmokeExplosion(phPara:GetGlobalPosition())
		
		phPara:Destroy()
		
		enemyGrp:SetSpeed(Vector2D(0, 400 + math.random() * 100))
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function bonusBase_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
			
	-- Smoke
	local phPara = enemyGrp:FindByName("phPara")
	if phPara ~= nil then
		CreateSmokeExplosion(phPara:GetGlobalPosition())
	end
	
	-- Bonus
	enemyGrp:GetUserData("OnReceiveBonusFn")()
	
	gameScreen:PlaySound(sndShieldBonus)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- grp
stockBonusBase = stockScreen:CreateGroup("stockBonusBase")
stockBonusBase:SetSpeed(Vector2D(0, 50))
stockBonusBase:SetUserData("points", 1)
stockBonusBase:SetUserData("enemyType", "bonus")

-- Functions
stockBonusBase:SetUserData("OnHitFn", bonusBase_OnHit)
stockBonusBase:SetUserData("OnShieldShockFn", bonusBase_OnShieldShock)

-- Parachute
tmpSprite = stockBonusBase:CreateSprite("sprPara")
tmpSprite:SetImage(parachuteTex)
tmpSprite.y = - tmpSprite:GetSize().y - 5
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
local radius = tmpSprite:GetSize().y / 2.5
tmp = stockBonusBase:CreatePhCircle(phGroupEnemies, "phPara", Vector2D(0, tmpSprite:GetPosition().x - radius), radius)
tmp:SetGlobalPosition(tmp:GetGlobalPosition())

-- Sprite
tmpSprite = stockBonusBase:CreateSprite("sprBidon")
tmpSprite:SetImage(bidonEnergyTex)
tmpSprite.y = - tmpSprite:GetSize().y / 2
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
local tmp = stockBonusBase:CreatePhRect(phGroupEnemies, "phBidon", tmpSprite:GetSize())
tmp:SetPosition(tmpSprite:GetPosition())

------------------------------------------------------------
-- Life Bonus
------------------------------------------------------------
stockEnergyBonus = stockBonusBase:Clone()
stockEnergyBonus:FindByNameSprite("sprBidon"):SetImage(bidonEnergyTex)
stockEnergyBonus:SetUserData("OnReceiveBonusFn",	function ()
														ChangeLife(200)
													end)

-- Register
addToSpawnList(	function (toGroup, forcedPos)
					bonusBase_Create(stockEnergyBonus, toGroup, forcedPos)
				end, 1, 0.2)

------------------------------------------------------------
-- Cash Bonus
------------------------------------------------------------
stockCashBonus = stockBonusBase:Clone()
stockCashBonus:FindByNameSprite("sprBidon"):SetImage(bidonCashTex)
stockCashBonus:SetUserData("OnReceiveBonusFn",	function ()
													ChangeCash(1000)
												end)

-- Register
addToSpawnList(	function (toGroup, forcedPos)
					bonusBase_Create(stockCashBonus, toGroup, forcedPos)
				end, 1, 0.1)

