------------------------------------------------------------
-- Missile alain
------------------------------------------------------------

-- Fns
function aMissile_Create(toGroup, forcedPos, missileLevel)
	if forcedPos == nil then
		forcedPos = Vector2D(math.random() * screenSize.x, -50)
	end
	
	local enemy = stockAMissile:CloneToGroup(toGroup)
	enemy:SetPosition(forcedPos)
	
	-- Speed
	enemy:SetSpeed(enemy:GetSpeed() * (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	-- Info
	missileLevel = math.min(missileLevel, #missileATexTbl)
	missileLevel = math.max(missileLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(missileATexTbl[missileLevel])
	enemy:SetUserData("life", missileLevel * 29)
	enemy:SetUserData("damage", 15 + missileLevel * 15)
	enemy:SetUserData("points", missileLevel)
	
	return enemy
end

function aMissile_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	enemyGrp:SetUserData("life", enemyGrp:GetUserData("life") - bulletGrp:GetUserData("damage"))

	if enemyGrp:GetUserData("life") <= 0 then -- If the enemy is dead
		-- Enemy fn
		-- Explosion
		CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		-- Points
		score = score + enemyGrp:GetUserData("points")
		
		enemyGrp:Destroy() -- We destroy the group
	else -- The enemy is style alive
		gameScreen:PlaySound(sndEnemyResist)
		
		-- Shield
		CreateSpriteFadeOutAnim(enemyGrp, missileAShieldTex:GetSize() / 2 * -1, missileAShieldTex, 1)
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function aMissile_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	
	-- Damage
	ChangeLife(- enemyGrp:GetUserData("damage"))
	CreateShieldImpact(hitPos.x)
	
	-- Explosion
	CreateExplosion(hitPos, math.pi, Vector2D(0, -1))
	gameScreen:PlaySound(sndShieldResist)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- Save
addToSpawnList(function(toGroup, forcedPos) aMissile_Create(toGroup, forcedPos, 1); end, 3, 1.5)
addToSpawnList(function(toGroup, forcedPos) aMissile_Create(toGroup, forcedPos, 2); end, 6, 1.5)
addToSpawnList(function(toGroup, forcedPos) aMissile_Create(toGroup, forcedPos, 3); end, 10, 1.5)
addToSpawnList(function(toGroup, forcedPos) aMissile_Create(toGroup, forcedPos, 4); end, 16, 2)

-- Missile
stockAMissile = stockScreen:CreateGroup("enemyAMissile")
stockAMissile:SetSpeed(Vector2D(0, 100))
stockAMissile:SetUserData("enemyType", "missile_a")

-- Functions
stockAMissile:SetUserData("OnHitFn", aMissile_OnHit)
stockAMissile:SetUserData("OnShieldShockFn", aMissile_OnShieldShock)

-- Sprite
tmpSprite = stockAMissile:CreateSprite("spr")
tmpSprite:SetImage(missileATexTbl[1])
tmpSprite.y = - tmpSprite:GetSize().y / 2
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
stockAMissile:CreatePhTriangle(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y / 2), Vector2D(tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y / 2), Vector2D(0, tmpSprite:GetSize().y / 2))


-- Reacteur --
model = ParticleModel(0.3, 0.8)
model:AddMutableParam("alpha", 0.7, 0)
model:AddMutableRandomParam("blue", 0.3, 0.5, 0, 0.1)
model:AddMutableRandomParam("red", 0.8, 1, 0.7, 1)
model:AddMutableRandomParam("green", 0.8, 1, 0, 0.2)
model:AddMutableRandomParam("size", 0.7, 1, 1.5, 2)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 4)
emitter:SetAngles(3.14 / 4, 0)
emitter:SetDirection(Vector2D(0, -1))

-- Particle
particleObj = stockAMissile:CreateParticleSource("particle", model, 24)
particleObj:MakeRenderImages(texParticlePoint, Vector2D(40, 40), true)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(30)
particleObj:SetForce(60, 200)
particleObj:SetFriction(3)
particleObj:SetGravity(Vector2D(0, -15))
particleObj.position = Vector2D(0, - tmpSprite:GetSize().y / 2 - 2)
particleObj.zPos = -100
